/*
 * Copyright 2016 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.uru;

import java.io.PrintStream;
import java.util.Arrays;

import at.jku.risc.stout.uru.algo.DebugLevel;
import at.jku.risc.stout.uru.algo.Substitution;
import at.jku.risc.stout.uru.algo.UnifAlgorithm;
import at.jku.risc.stout.uru.algo.UnifProblem;
import at.jku.risc.stout.uru.algo.UnifSystem;
import at.jku.risc.stout.uru.data.InputParser;
import at.jku.risc.stout.uru.data.atom.NodeFactory;

public class UnifyMain {

	static DebugLevel debugLevel = DebugLevel.SIMPLE;
	static boolean justifySigma = false;
	private static int maxDepth = 1000;

	public static void main(String[] args) {
		if (args.length >= 1) {
			if ("-d".equals(args[0])) {
				debugLevel = DebugLevel.PROGRESS;
				main(Arrays.copyOfRange(args, 1, args.length));
			} else if ("-h".equals(args[0])) {
				printHelp(System.out);
			} else if ("-j".equals(args[0])) {
				justifySigma = true;
				main(Arrays.copyOfRange(args, 1, args.length));
			} else if ("-m".equals(args[0])) {
				if (args.length < 2) {
					System.out.println("Insufficient parameters!");
					System.out.println();
					printHelp(System.out);
				} else {
					maxDepth  = Integer.parseInt(args[1]);
					main(Arrays.copyOfRange(args, 2, args.length));
				}
			} else {
				try {
					UnifProblem p = new UnifProblem();
					new InputParser(p, new NodeFactory()).parseEqSystem(args[0], null);
					new UnifAlgorithm(new UnifSystem(p, new Substitution()), maxDepth).unify(
							debugLevel, System.out, System.out, justifySigma);

					System.out.println();
					System.out
							.println("Supported by the Austrian Science Fund (FWF) "
									+ "under the project SToUT P 24087-N18");
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} else {
			System.out.println("Insufficient parameters!");
			System.out.println();
			printHelp(System.out);
		}
	}

	private static void printHelp(PrintStream out) {
		System.out.println("Sequence unification problem solver for unranked terms");
		System.out.println();
		System.out.println("Give a problem of the form:");
		System.out.println("\"s1 =? t1; s2 =? t2; ...; sn =? tn\"");
		System.out.println("where s1,...,sn and t1,...,tn are terms or hedges");
		System.out.println("Input example: \"f(X,Y) =? f(f(a))\"");
		System.out.println();
		System.out.println("Simple execution example: "
				+ "java at.jku.risc.stout.uru.UnifyMain -a "
				+ "\"f(a,X) =? f(a,b,c,d)\"");
		System.out.println("Advanced execution example with jvm parameters:");
		System.out.println("java -Xmx2048m -XX:MaxPermSize=1024m -Xss128m "
				+ "at.jku.risc.stout.uru.UnifyMain -j -m 10 -d "
				+ "\"f(a,X) =? f(a,b,c,d)\"");
		System.out.println();
		System.out.println("  -j    justify computed unifiers");
		System.out.println("  -m n  set maximum derivation depth to n");
		System.out.println("  -h    print this information");
		System.out
				.println("  -d    debug mode prints out every available information");
	}
}
